#include "cksm_list.h"

void init_list(memory_list * lst)
{
    INIT_LIST_HEAD(&lst->list);
}

void add_entry(memory_list * lst, int pid, unsigned long vm_start, unsigned long vm_end, int crc32)
{
    memory_list * entry = NULL;
    
    entry = (memory_list *) kmalloc(sizeof(memory_list), GFP_KERNEL);
    if(entry == NULL)
    {
        CKSMLOG("(%d) kmalloc failed\n", __LINE__);
        return;
    }
    
    entry->pid = pid;
    entry->vm_start = vm_start;
    entry->vm_end = vm_end;
    entry->crc32 = crc32;
    entry->is_present = 1;
    
    list_add(&(entry->list), &(lst->list));
}

void del_entry(memory_list * lst, int pid, unsigned long vm_start, unsigned long vm_end, int crc32)
{
    memory_list * entry;
    struct list_head * pos, * q;
    
    list_for_each_safe(pos, q, &lst->list)
    {
        entry = list_entry(pos, memory_list, list);
        
        if
        (
            (pid == 0 || entry->pid == pid)
            &&
            (vm_start == 0 || entry->vm_start == vm_start)
            &&
            (vm_end == 0 || entry->vm_end == vm_end)
            &&
            (crc32 == 0 || entry->crc32 == crc32)
        )
        {
            list_del(pos);
            kfree(entry);
            break;
        }
    }
}

void clean_list(memory_list * lst)
{
    memory_list * entry;
    struct list_head * pos, * q;
    
    list_for_each_safe(pos, q, &lst->list)
    {
        entry = list_entry(pos, memory_list, list);
        
        list_del(pos);
        kfree(entry);
    }
}

memory_list * find_entry(memory_list * lst, int pid, unsigned long vm_start, unsigned long vm_end)
{
    memory_list * entry;
    struct list_head * pos;
    
    list_for_each(pos, &lst->list)
    {
        entry = list_entry(pos, memory_list, list);
        if
        (
            entry->pid == pid
            &&
            entry->vm_start == vm_start
            &&
            entry->vm_end == vm_end
        )
        {
            return entry;
        }
    }
    
    return NULL;
}

void show_list(memory_list * lst)
{
    memory_list * entry;
    struct list_head * pos;
    
    list_for_each(pos, &lst->list)
    {
        entry = list_entry(pos, memory_list, list);
        
        CKSMLOG
        (
            "(%d) pid=%d, vm_start=0x%lx, vm_end=0x%lx, crc32=%08X, is_present=%08X\n",
            __LINE__,
            entry->pid,
            entry->vm_start, entry->vm_end,
            entry->crc32,
            entry->is_present
        );
    }
}

void del_unused_entries(memory_list * lst, int pid)
{
    memory_list * entry;
    struct list_head * pos, * q;
    
    list_for_each_safe(pos, q, &lst->list)
    {
        entry = list_entry(pos, memory_list, list);
        
        if(entry->pid == pid && !entry->is_present)
        {
            del_entry(lst, entry->pid, entry->vm_start, entry->vm_end, entry->crc32);
            CKSMLOG
            (
                "(%d) deleted: pid=%d, vm_start=0x%lx, vm_end=0x%lx, crc32=%08X\n",
                __LINE__,
                entry->pid,
                entry->vm_start, entry->vm_end,
                entry->crc32
            );
        }
    }
}

void zero_is_present(memory_list * lst, int pid)
{
    memory_list * entry;
    struct list_head * pos;
    
    list_for_each(pos, &lst->list)
    {
        entry = list_entry(pos, memory_list, list);
        
        if(entry->pid == pid)
            entry->is_present = 0;
    }
}
