#include <linux/module.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <asm/uaccess.h>

#include "cksm_func.h"

#define DEV_NAME "testdev"

#define IOCTL_SAMPLE _IOWR(800, 0, int)

static int device_open(struct inode *, struct file *);
static int device_release(struct inode *, struct file *);
static long device_ioctl(struct file * file, unsigned int ioctl_num, unsigned long ioctl_param);

static int major;
static int is_dev_open = 0;

static struct file_operations fops =
{
    .open = device_open,
    .unlocked_ioctl = device_ioctl,
    .release = device_release
};


static long device_ioctl(struct file * file, unsigned int ioctl_num, unsigned long ioctl_param)
{
    int result;
    
    switch(ioctl_num)
    {
        case IOCTL_SAMPLE:
            printk(KERN_ALERT "IOCTL_SAMPLE\n");
            
            result = cksm_check();
            
            put_user(result, (int *)ioctl_param);
        break;
    }

    return 0;
}

static int __init mymodule_init(void)
{
    printk("Module init\n");
    
    major = register_chrdev(0, DEV_NAME, &fops);
    if(major < 0)
    {
        printk(KERN_ALERT "register_chrdev (%d)\n", major);
        return major;
    }
    
    cksm_init();

    printk(KERN_INFO "'mknod /dev/%s c %d 0'\n", DEV_NAME, major);    

    return 0;
}

static void __exit mymodule_exit(void)
{
    printk("Module exit\n");
    
    unregister_chrdev(major, DEV_NAME);
    
    cksm_fini();
}

static int device_open(struct inode * inode, struct file * file)
{
    printk("Device open\n");
    
    if(is_dev_open)
        return -EBUSY;

    is_dev_open++;
    
    try_module_get(THIS_MODULE);

    return 0;
}

static int device_release(struct inode * inode, struct file * file)
{
    printk("Device release\n");
    
    is_dev_open--;
    module_put(THIS_MODULE);

    return 0;
}


module_init(mymodule_init);
module_exit(mymodule_exit);

MODULE_LICENSE("GPL");
